/*
 * @(#)RussianKeyboard.java	Jan-2008
 *
 * Murat iNAN
 */
package com.javamar.klavye;

import java.awt.*;
import java.awt.font.*;
import java.awt.image.*;
import java.awt.event.*;

import java.io.*;

import java.text.*;

import java.util.*;

import javax.swing.*;
import javax.swing.text.*;
import javax.swing.border.*;

//custom
import com.javamar.arac.*;
import com.javamar.filtre.*;


/**
 * This class wrote for solving Russian characters in latin
 * keymaps. 
 *
 * @author Murat iNAN
 * <p>
 * <a href=http://www.dilmerkezi.net>
 * http://www.dilmerkezi.net
 * </a>
 */
public 	class 		RussianKeyboard
	extends		JPanel
	implements 	ActionListener, Serializable
{

    /** Lang is Turkish*/
    private static boolean turkish = false;
    
    static
    {
            java.util.Locale l = java.util.Locale.getDefault ();
	    String lang = l.getLanguage ();
	    lang = lang.toLowerCase ();
	    if (lang.equals ("tr"))
	    {
	            turkish = true;
	    }
	    System.out.println ("\nLang is Turkish: "+turkish+"");
    }
     
    /** White constant*/
    private static final Color WHITE = Color.WHITE;
    
    /** Predefined bg color*/
    private static Color bgcolor = WHITE;
    
    /** Blue constant*/
    private static final Color BLUE = Color.BLUE;
    
    /** Predefined fg color*/
    private static Color fgcolor = BLUE;
    
    /** Green constant*/
    private static final Color GREEN = Color.green.darker ();
    
    /** Orange constant*/
    private static final Color ORANGE = Color.orange.darker ();    
    
    /** Red constant*/
    private static final Color RED = Color.red;
    
    /** Black constant*/
    private static final Color BLACK = Color.black;
    
    /** Magenta constant*/
    private static final Color MAGENTA = Color.magenta;
        
    /** Lucida Font constant*/
    private static final Font LUCIDA = new Font ("Lucida Sans Regular", Font.PLAIN, 14);

    /** Predefined font*/
    private static Font font = LUCIDA;
    
    /** Monospaced font*/
    private static final Font fontMONO = new Font ("Monospaced", 0, 10);
    
    /** Lucida Bold Font constant*/
    private static final Font LUCIDABOLD = new Font ("Lucida Typewriter Bold", Font.BOLD, 14);

    /** Scheherazade Font constant*/
    private static final Font SCHEHERAZADE = new Font ("Scheherazade", 0, 26);
    
    /** Other Scheherazade Font constant*/
    private static final Font SCHEHERAZADE_NW = new Font ("Scheherazade", 1, 16);
    
    /** Menu items dimension constant*/
    private static final Dimension DIMENSION = new Dimension (125, 32);
    
    /** Menubar*/
    private static final JMenuBar mb = new JMenuBar ();

    /** File menu*/
    private static final JMenu fileMenu = getMenu ("File", "Dosya");

    /** Color menu*/
    private static final JMenu colorMenu = getMenu ("Color", "Renk");

    /** Font menu*/
    private static final JMenu fontMenu = getMenu ("Font", "Yaz\u0131");

    /** Ottoman characters menu*/
    private static final JMenu osmanliMenu  = getMenu ("Osmanl\u0131ca", "Osmanl\u0131ca");
    
    /** Process menu, stop-gc-exit*/
    private static final JMenu processMenu = getMenu ("Process", "S\u00FCre\u00E7");

    /** Document*/
    private static StyledDocument doc = new XPDocument ();
    
    /** Text Pane for inserting characters*/
    private static final JTextPane txt = new MTextPane (); 
    
    /** Initialize flag*/
    private static boolean initialize = false;

    /** New file Item*/
    private static final JMenuItem newItem = getItem ("New", "Yeni");
    
    /** Open single text files*/
    private static final JMenuItem sopenItem = getItem ("Open S", "Metin A\u00E7");
    
    /** Open file Item*/
    private static final JMenuItem openItem = getItem ("Open U", "A\u00E7 U");
    
    /** Save file Item*/
    private final static JMenuItem saveItem = getItem ("Save  U", "Kaydet U");
    
    /** Exit Item*/
    private final static JMenuItem exitItem = getItem ("Exit", "\u00C7\u0131k");

    /** Text pane background color*/
    private final static JMenuItem bgItem = getItem ("BG", "BG");
    
    /** Text pane font color*/
    private final static JMenuItem fgItem = getItem ("FG", "FG");
    	
    /** Font family*/    
    private final static JMenuItem familyItem = getItem ("Family", "Aile");
    
    /** Font size*/
    private final static JMenuItem sizeItem = getItem ("Size", "Boyut");
    
    /** Font style like bold, itakic etc.*/
    private final static JMenuItem styleItem = getItem ("Style", "Stil");

    /** Stop Item*/
    private final static JMenuItem stopItem = getItem ("Stop", "Dur");
    
    /** Garbage collector*/
    private final static JMenuItem gcItem = getItem ("GC", "GC");
    
    /** Clear text of text area*/
    private final static JMenuItem clearTextItem = getItem ("Clear Text", "Metni Sil");
    
    /** Clear text of status bar*/
    private final static JMenuItem clearStatusItem = getItem ("Clear Status", "Durum Sil");
    
    /** Stop flag*/
    private static boolean stop = false;
    
    /** File Chooser Tool*/
    private static final JFileChooser jfc = new JFileChooser ();

    /** Second File Chooser*/
    private static final JFileChooser jfc_arab = new JFileChooser ();
    
    /** Single text files*/
    private static final JFileChooser singlejfc = new JFileChooser ();
    
    /** Info Label*/
    private final static JLabel infoLabel = new JLabel ("", JLabel.LEFT);
    
    /** Obtain Russian characters*/
    private static final String [] arstr = Tool.getCyrillicstr (); 

    /** The length of Russian characters*/
    private static final int salen = arstr.length;

    /** For memory control*/
    private static final Runtime rt = Runtime.getRuntime ();
    
    /** Main char panel*/
    private static JPanel letters;
    
    /** Arabic chars panel 1*/
    private static JPanel zupperpanel;
    
    /** Arabic chars panel 2*/
    private static JPanel zlowerpanel;
    
    /** Center panel*/
    private static JPanel cpanel;
    
    /** Main frame*/
    private static JFrame fr;
    
    
    /** Singleton Constructor*/
    private RussianKeyboard ()
    {
	super (new BorderLayout ());
	
	letters = new JPanel (new GridLayout (0, 2, 2, 0));
	
	JButton spaceX = getButton ("Space", "Bo\u015Fluk");
	JButton nlX = getButton ("NL", "NL");	
	
	cpanel = new JPanel (new GridLayout (2, 0, 1, 1));
	
	/////////////////////
	zupperpanel = new JPanel (new GridLayout (5, 5, 1, 1));
	zupperpanel.setBackground (WHITE);
	zupperpanel.setBorder (BorderFactory.createTitledBorder (
	BorderFactory.createLineBorder (BLUE, 1),
	    turkish?"Karakterler":"Characters",
	    TitledBorder.LEFT,
	    TitledBorder.TOP,
	    LUCIDA,
	    RED
	));

	final String [] zussrstrs = Tool.getCyrillicstr ();
	final int zslen = zussrstrs.length;

	for (int i = 0; i < 33; i++)
	{
	    JButton b = getButtonX (zussrstrs [i], zussrstrs [i]);
	    b.addActionListener (this);
	    zupperpanel.add (b);
	}

	zupperpanel.add (spaceX);
	zupperpanel.add (nlX);

	zlowerpanel = new JPanel (new GridLayout (5, 5, 1, 1));
	zlowerpanel.setBackground (WHITE);
	zlowerpanel.setBorder (BorderFactory.createTitledBorder (
	BorderFactory.createLineBorder (BLUE, 1),
	    turkish?"Di\u011Fer Karakterler":"Other Characters",
	    TitledBorder.LEFT,
	    TitledBorder.TOP,
	    LUCIDA,
	    RED
	));

	for (int i = 33; i < zslen; i++)
	{
	    JButton b = getButton (zussrstrs [i], zussrstrs [i]);
	    b.addActionListener (this);
	    zlowerpanel.add (b);
	}

	letters.add (zupperpanel);
	letters.add (zlowerpanel);
	cpanel.add (letters);
	////////////////////

	jfc.setAcceptAllFileFilterUsed (false);
	singlejfc.setAcceptAllFileFilterUsed (false);
	
	jfc.addChoosableFileFilter (new Filter ());
	singlejfc.addChoosableFileFilter (new FilterFN ());
	
	infoLabel.setForeground (BLUE);
	infoLabel.setFont (LUCIDA);
	
	mb.setBackground (WHITE);
	
	JPanel npanel = new JPanel (new BorderLayout ());
	npanel.setPreferredSize (new Dimension (770, 35));
	npanel.add ("Center", mb);
	
	add ("North", npanel);
	
	mb.add (fileMenu);
	
	fileMenu.setMnemonic (KeyEvent.VK_ALT);
	
	fileMenu.add (newItem);
	newItem.setAccelerator ( KeyStroke.getKeyStroke (
	    KeyEvent.VK_N, ActionEvent.CTRL_MASK) );
	newItem.addActionListener (new ActionListener ()
	{
	    public void actionPerformed (ActionEvent evt)
	    {
		try
		{
		    infoLabel.setText ("");
		    stop = true;
		    Thread.sleep (200);
		    stop = false;
		}
		catch (Exception e)
		{}
		
		try
		{
		    doc = new XPDocument ();
		    txt.setDocument (doc);
		}
		catch (Exception e)
		{}
	    }
	});
	
	fileMenu.addSeparator ();
	
	fileMenu.add (sopenItem);
	sopenItem.setAccelerator ( KeyStroke.getKeyStroke (
	    KeyEvent.VK_T, ActionEvent.CTRL_MASK) );	
	sopenItem.addActionListener (new ActionListener ()
	{
	    public void actionPerformed (ActionEvent evt)
	    {
		infoLabel.setText ("");
		sopen ();
	    }
	});	
	
	fileMenu.addSeparator ();
	
	fileMenu.add (openItem);
	openItem.setAccelerator ( KeyStroke.getKeyStroke (
	    KeyEvent.VK_O, ActionEvent.CTRL_MASK) );	
	openItem.addActionListener (new ActionListener ()
	{
	    public void actionPerformed (ActionEvent evt)
	    {
		infoLabel.setText ("");
		open ();
	    }
	});

	fileMenu.add (saveItem);
	saveItem.setAccelerator ( KeyStroke.getKeyStroke (
	    KeyEvent.VK_S, ActionEvent.CTRL_MASK) );
	saveItem.addActionListener (new ActionListener ()
	{
	    public void actionPerformed (ActionEvent evt)
	    {
		infoLabel.setText ("");
		save ();
	    }
	});

	fileMenu.addSeparator ();

	exitItem.setAccelerator ( KeyStroke.getKeyStroke (
	    KeyEvent.VK_Q, ActionEvent.CTRL_MASK) );
	fileMenu.add (exitItem);

	exitItem.addActionListener (new ActionListener ()
	{
	    public void actionPerformed (ActionEvent evt)
	    {
		System.exit (-1);
	    }
	});
	
	mb.add (colorMenu);
	
	colorMenu.add (bgItem);
	bgItem.addActionListener (new ActionListener ()
	{
	    public void actionPerformed (ActionEvent evt)
	    {
		Color c = JColorChooser.showDialog (txt, "BG color", WHITE);
		if (c != null)
		{
		    bgcolor = c;
		    txt.setBackground (bgcolor);
		}
	    }
	});
	
	colorMenu.add (fgItem);
	fgItem.addActionListener (new ActionListener ()
	{
	    public void actionPerformed (ActionEvent evt)
	    {
		Color c = JColorChooser.showDialog (txt, "FG color", BLUE);
		if (c != null)
		{
		    fgcolor=c;
		    if (doc.getLength () < 1)
		    {
			try
			{
			    doc.insertString (0, "a", null);
			    txt.setForeground (fgcolor);
			    doc.remove (0, 1);
			    return; 			    
			}
			catch (Exception e)
			{
			}
		    }
		    txt.setForeground (fgcolor);
		}
	    }
	});	

	mb.add (fontMenu);
	fontMenu.add (familyItem);
	familyItem.addActionListener (new ActionListener ()
	{
	    public void actionPerformed (ActionEvent evt)
	    {
		Font f = Tool.getFamiliedFont (txt);
		if (f != null)
		{
		    font = f;
		    txt.setFont (font);
		}
	    }
	});
	
	fontMenu.add (sizeItem);
	sizeItem.addActionListener (new ActionListener ()
	{
	    public void actionPerformed (ActionEvent evt)
	    {
		Font f = Tool.getSizedFont (txt);
		if (f != null)
		{
		    font = f;
		    txt.setFont (font);
		}
	    }
	});	
	
	fontMenu.add (styleItem);
	styleItem.addActionListener (new ActionListener ()
	{
	    public void actionPerformed (ActionEvent evt)
	    {
		Font f = Tool.getStyledFont (txt);
		if (f != null)
		{
		    font = f;
		    txt.setFont (font);
		}
	    }
	});

	mb.add (processMenu);
	
	processMenu.add (stopItem);
	
	stopItem.addActionListener (new ActionListener ()
	{
	    public void actionPerformed (ActionEvent evt)
	    {
		stop = (!(stop));
		infoLabel.setText ("");
		if (stop)
		{
		    stopItem.setText ("Start");
		}
		else
		{
		    stopItem.setText ("Stop");		
		}
	    }
	});
	
	processMenu.add (gcItem);
	gcItem.addActionListener (new ActionListener ()
	{
	    public void actionPerformed (ActionEvent evt)
	    {
		infoLabel.setText ("");
		stop = true;
		try
		{
		    Thread.sleep (100);
		}
		catch (Exception e)
		{
		    stop = false;
		}
		infoLabel.setText ("Please wait while virtual memory freeing");
		int before = (int)(rt.freeMemory ());
		StringBuffer sb = new StringBuffer ();
		sb.append ("Free jv mem before: "+before+ " B");
		double [] d = new double [1500000];
		System.gc ();
		before = (int)(rt.freeMemory ());
		sb.append ("; After: "+before+" B");
		infoLabel.setText (sb.toString ());
		stop = false;
	    }
	});
	
	processMenu.addSeparator ();
	
	processMenu.add (clearTextItem);	
	clearTextItem.addActionListener (new ActionListener ()
	{
	    public void actionPerformed (ActionEvent evt)
	    {
		infoLabel.setText ("");
		txt.setText ("");
	    }
	});	
	
	processMenu.add (clearStatusItem);
	clearStatusItem.addActionListener (new ActionListener ()
	{
	    public void actionPerformed (ActionEvent evt)
	    {
		infoLabel.setText ("");
	    }
	});

	JScrollPane jsp = new JScrollPane (
	    ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
	    ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
	JViewport port = jsp.getViewport ();
	jsp.setBackground (WHITE);
	jsp.setBorder (BorderFactory.createTitledBorder (
	    BorderFactory.createLineBorder (BLUE, 1),
	    turkish?"Metin Alan\u0131":"Text Area",
	    TitledBorder.LEFT,
	    TitledBorder.TOP,
	    LUCIDA,
	    RED
	));	
	
	port.add (txt);
	
	txt.setDocument (new XPDocument ());
	doc = txt.getStyledDocument ();	
	
	spaceX.addActionListener (new ActionListener ()
	{
	    public void actionPerformed (ActionEvent evt)
	    {
		try
		{
		    doc.insertString (txt.getCaretPosition (), " ", null);
		}
		catch (Exception e)
		{}    
	    }
	});
	
	nlX.addActionListener (new ActionListener ()
	{
	    public void actionPerformed (ActionEvent evt)
	    {
		try
		{
		    doc.insertString (txt.getCaretPosition (), "\n", null);
		}
		catch (Exception e)
		{}    
	    }
	});	
	
	cpanel.add (jsp);
	
	add ("Center", cpanel);
	
	JPanel spanel = new JPanel (new BorderLayout ());
	spanel.setBackground (WHITE);
	spanel.setBorder (BorderFactory.createLineBorder (GREEN, 1));
	spanel.setPreferredSize (new Dimension (790, 35));
	
	spanel.add ("Center", infoLabel);
	
	add ("South", spanel);
    } //End of constructor
    
    /**
     * @param mw Width of image
     * @param mh Height of image
     * @return If there is memory enough 1, otherwise (-1);
     */
    private int memcontrol (int mw, int mh)
    {
	final int zwh = (mw*mh)+1024;
	int mem = (int)(rt.freeMemory ());
	if (zwh >= mem)
	{
	    infoLabel.setText ("There is not enough memory for now. Try again");
	    return (-1);
	}
	return (1);
    }

    /**
     * Get custom menus
     *
     * @param str The text of menu
     * @param  tr The Turkish text of menu     
     * @return A suitable custom menu
     */
    private final static JMenu getMenu (String str, String tr)
    {
	JMenu m = new JMenu (turkish?tr:str);
	m.setBackground (WHITE);
	m.setForeground (BLUE);
	m.setFont (LUCIDA);
	return m;
    }

    /**
     * Get custom menu items
     *
     * @param str The text of menuitem
     * @param  tr The Turkish text of menuitem     
     * @return A suitable custom menuitem
     */
    private final static JMenuItem getItem (String str, String tr)
    {
	JMenuItem m = new JMenuItem (turkish?tr:str);
	m.setPreferredSize (DIMENSION);
	m.setBackground (WHITE);
	m.setForeground (BLUE);
	m.setFont (LUCIDA);
	return m;
    }
    
    /**
     * Get custom menu items
     *
     * @param str The text of menuitem
     * @param  tr The Turkish text of menuitem     
     * @return A suitable custom menuitem
     */
    private final static JMenuItem getItemX (String str, String tr)
    {
	JMenuItem m = new JMenuItem (turkish?tr:str);
	m.setBackground (WHITE);
	m.setForeground (BLUE);
	m.setFont (LUCIDA);
	return m;
    }    
    
    /**
     * Get custom buttons
     *
     * @param str The text of button
     * @param  tr The Turkish text of button     
     * @return A suitable custom button
     */
    private final static JButton getButton (String str, String tr)
    {
	JButton m = new JButton (turkish?tr:str);
	m.setBackground (WHITE);
	m.setForeground (BLUE);
	m.setFont (LUCIDABOLD);
	return m;
    }    
    
    /**
     * Get custom buttons
     *
     * @param str The text of button
     * @param  tr The Turkish text of menuitem     
     * @return A suitable custom button
     */
    private final static JButton getButtonX (String str, String tr)
    {
	JButton m = new JButton (turkish?tr:str);
	m.setBackground (WHITE);
	m.setForeground (BLUE);
	return m;
    }       
    
    ////// INNER CLASS ////////////
    private final static 
	    class	MTextPane 
	    extends	JTextPane
    {
	/**    
         *
         * Get custom text pane
         *
         * @return A suitable custom text pane
         */
	private MTextPane ()
	{
	    super ();
	    setDocument (new XPDocument ());
    	    setFont (fontMONO);
	    setBackground (bgcolor);
    	    setForeground (fgcolor);
    	    setDragEnabled (true);
	}
	
	/**
	 * Line wrap or no?
	 * @return If wrap true, otherwise false
	 */ 
	public boolean getScrollableTracksViewportWidth ()
	{
	    return false;
	}
	
	/** @return a dimension*/
	public Dimension getPreferredSize ()
	{
	    Dimension d = super.getPreferredSize ();
	    if (this.getParent () instanceof JViewport)
	    {
		JViewport p = (JViewport)(this.getParent ());
		javax.swing.plaf.TextUI ui = this.getUI ();
		int w = d.width;
		int h = d.height;
		if (this.getScrollableTracksViewportWidth () == false)
		{
		    int wx = p.getWidth ();
		    Dimension max = ui.getMaximumSize (this);
		    if (wx != 0 && wx < max.width)
		    {
			w = 1600; //more than viewport size
		    }
		}
		if (w != d.width || h != d.height)
		{
		    d = new Dimension (w, h);
		}
	    }
	    return d;
	}
	
	/**
	 * @param g Graphics object
	 */
	protected void paintComponent (Graphics g)
	{
	    super.paintComponent (g);
	}
	
    }
    /////////////////////
    
    /////////////////////////
    private final
    class	ExtraActionListener
    implements	ActionListener
    {
    
	/** 
        * Manage actions here
        *
        * @param evt ActionEvent object
        */
        public void actionPerformed (ActionEvent evt)
	{
	    Object obj = evt.getSource ();
	    if (obj instanceof JMenuItem)
	    {
		JMenuItem b = (JMenuItem)(obj);
		if (b != null)
		{
		    String text = b.getText ();
		    if (text != null)
		    {
			try
			{
			    doc.insertString (txt.getCaretPosition (),
			    text, null);
			}
			catch (Exception ex)
			{}
		    }
		}
	    }
	}
    
    } //end of ExtraActionListener
    //////////////////
    
    /**
     * Open desired file and write its content to
     * the text area
     *
     * @return If successfull true otherwise false
     */
    private final boolean open ()
    {
	//if (stop) return false;
	
	infoLabel.setText ("");
	
	doc = txt.getStyledDocument ();
	if (doc == null) return false;
	
	stop = true;
	
	try
	{
	    Thread.sleep (100);
	}
	catch (Exception e)
	{}
	
	stop = false;
	
	Thread thr = new Thread ()
	{
	    public void run ()
	    {
		int rep = jfc.showOpenDialog (txt);
		if (rep != JFileChooser.APPROVE_OPTION)
		{
		    infoLabel.setText ("Cancelled");
		    stop = true;
		    return;    
		}
		
		File f = jfc.getSelectedFile ();
		if (f == null || f.exists () == false)
		{
		    infoLabel.setText (f.getName () + " is null");
		    stop = true;
		    return;
		}
		
		DataInputStream dis = null;
		try
		{
		    dis = new DataInputStream (
			new BufferedInputStream (
			new FileInputStream (f)));
		}
		catch (Exception ex)
		{
		    infoLabel.setText (ex.getMessage () + " error");
		    stop = true;
		    return;
		}
		
		if (dis == null)
		{
		    infoLabel.setText ("Stream is null error");
		    stop = true;
		    return;
		}
		
		String line = null;
		
		boolean error = false;
		
		txt.setText ("");
		
		try
		{
		    while ( (!stop) && 
			    (!(error)) && 
			    (line=dis.readUTF ()) != null)
		    {
			try
			{
			    doc.insertString (doc.getLength (), line, null);
			}
			catch (Exception strex)
			{
			    continue;
			}
		    }
		}
		catch (Exception ex)
		{
		    error = true;
		}
		
		try
		{
		    dis.close ();
		}
		catch (Exception e)
		{
		    stop = true;
		    return;
		}
		infoLabel.setText ("Opened " + f.getName () + "");		
    		stop = false;
	    } //run
	}; //thr
	
	thr.start ();
	return true;
    }
    
    /**
     * Opens single text files
     */
    private final boolean sopen ()
    {
	Thread thr = new Thread ()
	{
	    public void run ()
	    {	
		try
		{
		    int rep = singlejfc.showOpenDialog (mb);
		    if (rep != JFileChooser.APPROVE_OPTION)
		    {
			stop = true;
			return;
		    }
		    File f = singlejfc.getSelectedFile ();
		    BufferedReader buf = new BufferedReader (
			new FileReader (f));
		    String line = null;
		    boolean error = false;
		    try
		    {
			txt.setText ("");
			while ( (!error) && (!stop) && (line=buf.readLine ()) != null )
			{
			    doc.insertString (doc.getLength (), line+"\n", null);
			}
		    }
		    catch (Exception lex)
		    {
			error = true;
		    }
		    buf.close ();
		    infoLabel.setText ("Opened " + f.getName () + "");		
    		    stop = false;		    
		}
		catch (Exception ex)
		{
		    stop = true;
		    return;
		}
	    } //run
	}; //thr
	thr.start ();
	return true;
    }
    
    /**
     * Save the contents of text area to desired file
     *
     * @return If successfull true otherwise false
     */    
    private final boolean save ()
    {
	//if (stop) return false;

	infoLabel.setText ("");	
	
	stop = true;
	try
	{
	    Thread.sleep (100);
	}
	catch (Exception e)
	{}	
	
	stop = false;
	
	Thread thr = new Thread ()
	{
	    public void run ()
	    {
		int rep = jfc.showSaveDialog (txt);
		if (rep != JFileChooser.APPROVE_OPTION)
		{
		    infoLabel.setText ("Cancelled");
		    stop = true;
		    return;    
		}
		
		File f = jfc.getSelectedFile ();
		
		String name = f.getName ().toLowerCase ();
		if (name.endsWith (".rus") == false)
		{
		    f = new File (f.getAbsolutePath ()+".rus");
		}
		
		if (f.exists ())
		{
		    infoLabel.setText (f.getName () + " exists. Select another file");
		    stop = true;
		    return;
		}
		
		DataOutputStream dos = null;
		try
		{
		    dos = new DataOutputStream (
			new BufferedOutputStream (
			new FileOutputStream (f)));
		}
		catch (Exception ex)
		{
		    infoLabel.setText (ex.getMessage () + " error");
		    stop = true;
		    return;
		}
		
		if (dos == null)
		{
		    infoLabel.setText ("Stream is null error");		
		    stop = true;
		    return;
		}
		
		String line = null;
		
		BufferedReader buf = null;
		
		try
		{
		    buf = new BufferedReader (
			new StringReader (
			txt.getText ()));
		}
		catch (Exception ex)
		{
		    try
		    { 
			dos.close ();
		    }
		    catch (Exception r)
		    {}
		    infoLabel.setText (ex.getMessage () + " error");
		    stop = true;
		    return;
		}
		
		boolean error = false;
		
		try
		{
		    while ( (!stop) && 
			    (!(error)) && 
			    (line=buf.readLine ()) != null)
		    {
			try
			{
			    dos.writeUTF (line +"\n");
			}
			catch (Exception strex)
			{
			    continue;
			}
		    }
		}
		catch (Exception ex)
		{
		    error = true;
		}
		
		try
		{
		    dos.close ();
		}
		catch (Exception e)
		{
		    stop = true;
		    return;
		}
		
		infoLabel.setText ("Saved " + f.getName () + "");
		stop = false;
	    } //run
	}; //thr
	
	thr.start ();
	return true;
    }
    
    /**
     * Manage actions here
     *
     * @param evt ActionEvent object
     */
    public void actionPerformed (ActionEvent evt)
    {
	infoLabel.setText ("");
	Object obj = evt.getSource ();
	if (obj instanceof JButton)
	{
	    JButton b = (JButton)(obj);
	    if (b != null)
	    {
		String text = b.getText ();
		if (text != null)
		{
		    try
		    {
			doc.insertString (txt.getCaretPosition (),
			 text, null);
		    }
		    catch (Exception ex)
		    {}
		}
	    }
	}
    }
    
    /** Show main frame*/
    private static final void showFrame ()
    {
	String osname = System.getProperty ("os.name");
	if (osname.indexOf ("Linux") != -1)
	{ //Use Linux
	    try
	    {
		javax.swing.plaf.metal.MetalLookAndFeel.setCurrentTheme (
	    	    new javax.swing.plaf.metal.DefaultMetalTheme ());
		UIManager.setLookAndFeel (
		    new javax.swing.plaf.metal.MetalLookAndFeel ()); 
	    //JFrame.setDefaultLookAndFeelDecorated (true);
	    //JDialog.setDefaultLookAndFeelDecorated (true);
	    }
	    catch (Exception e)
	    {}
	}
	else if (osname.indexOf ("Windows") != -1)
	{ //Windows
	    try
	    {
		UIManager.setLookAndFeel (
		    "com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
	    }
	    catch (Exception ex)
	    {}	
	}
	else
	{ //Other like Mac etc
	    try
	    {
		UIManager.setLookAndFeel (UIManager.getSystemLookAndFeelClassName ());
	    }
	    catch (Exception ex)
	    {}
	}
	
	//Get frame
	fr = new JFrame ("Russian Keyboard, www.dilmerkezi.net");
	fr.setDefaultCloseOperation (JFrame.EXIT_ON_CLOSE);
	
	//Mainboard of frame
	JComponent jc = new RussianKeyboard ();
	jc.setOpaque (true);
	fr.setContentPane (jc);

	try
	{
	    javax.swing.SwingUtilities.updateComponentTreeUI (fr);	
	    javax.swing.SwingUtilities.updateComponentTreeUI (jfc);
	    javax.swing.SwingUtilities.updateComponentTreeUI (jfc_arab);
	    javax.swing.SwingUtilities.updateComponentTreeUI (singlejfc);	    
	}
	catch (Exception j)
	{}
	
	//Size frame and show it
	fr.pack ();
	fr.setSize (800, 600);
	//fr.setResizable (false);
	fr.setMinimumSize (new Dimension (800, 600));
	fr.setMaximumSize (new Dimension (1024, 768));
	fr.setLocationRelativeTo (null);
	fr.setVisible (true);	
    }
    
    /**
     * Show message and exit
     *
     * @param msg Message for shown
     */
    private static void debugmsgexit (String msg)
    {
	JOptionPane.showMessageDialog (null, msg);
	System.exit (-1);
    }
    
    /**
     * Main method. Applications call this method first.
     *
     * @param args String array
     */
    public static void main (String [] args)
    {
    
	if (initialize)
	{
	    debugmsgexit ("Error: Program already running.\nExiting...");
	}
	
	initialize = true;
	
	javax.swing.SwingUtilities.invokeLater (new Runnable ()
	{
	    public void run ()
	    { 	//Call showFrame in thread
		showFrame ();
	    }
	});
    }
    
}
/*************/

